@@ -143,7 +143,12 @@ class Connection(object):
             if not authSession:
                 raise ValueError("invalid username or password")
 
-            self._client.setSchema(database)
+            buffer = self._client.isSchemaSet()
+            buffer = BytesIO(buffer)
+            dis = DataInputStream(buffer)
+            is_schema_set = dis.read_boolean()
+            if not is_schema_set:
+                self._client.setSchema(database)
         except Exception as e:
             self._transport.close()
             raise e
@@ -156,7 +161,8 @@ class Connection(object):
         """Call close"""
         self.close()
 
-    def close(self):
+    def close(self, query_id):
+        self._client.clear(query_id)
         self._transport.close()
 
     def reopen(self):
@@ -253,6 +259,7 @@ class Cursor(common.DBAPICursor):
 
     def close(self):
         """Close the operation handle"""
+        self.connection.close(self._query_id)
         pass
 
     def execute(self, operation, parameters=None, **kwargs):
@@ -361,9 +368,12 @@ if __name__ == '__main__':
     cursor.execute("select * from date_dim")
     cursor.connection.reopen()
     bdata = cursor._fetch_more()
-    while bdata:
-        print(len(bdata))
-        bdata = cursor._fetch_more()
+    try:
+        while bdata:
+            print(len(bdata))
+            bdata = cursor._fetch_more()
+    except Exception as e:
+        cursor.close()
 
 #
 # Type Objects and Constructors
